using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Shapes;
using System.Collections.Generic;

namespace AvFilter3x3
{ public partial class MainPage:UserControl 
  { int xSize, ySize;
    double dx, dy;
    bool MouseButtonPressed = false;
    Byte[,] image0, image1; // input and output images
    List<SolidColorBrush> ShadesOfGray = new List<SolidColorBrush>();
    public MainPage() 
    { InitializeComponent();
      for ( int i=0; i < 10; i++ ) //create 10 brushes
      { Byte b = (Byte)(i*25);     //10 grades of gray 0, 25, 50, ... 250
        ShadesOfGray.Add( new SolidColorBrush( Color.FromArgb( (Byte)255, b, b, b ) ) );
      }
    }
    private void canvas_Loaded( object sender, RoutedEventArgs e ) 
    { DrawSomething.Visibility = Visibility.Visible;
      Size_ValueChanged( null, null );
    }
    private void Size_ValueChanged( object sender, RoutedPropertyChangedEventArgs<double> e )
    { try { xSize = Convert.ToInt32( xSizeScrollBar.Value ); } catch { return; }
      try { ySize = Convert.ToInt32( ySizeScrollBar.Value ); } catch { return; }
      xSizeTextBlock.Text = xSize.ToString();
      ySizeTextBlock.Text = ySize.ToString();
      dx = DrawCanvas.Width / xSize;
      dy = DrawCanvas.Height/ ySize;
      GridLines.Points.Clear();
      //meandring red grid lines
      for ( int x=0; x < xSize-1; x+=2 )  //2 vertical lines per step
      { GridLines.Points.Add( new Point( x*dx     ,        0 ) ); //start at upper border
        GridLines.Points.Add( new Point( x*dx     , ySize*dy ) ); //go down to lower border
        GridLines.Points.Add( new Point( x*dx+  dx, ySize*dy ) ); //go 1 right at lower border
        GridLines.Points.Add( new Point( x*dx+  dx,        0 ) ); //go up to upper border
        GridLines.Points.Add( new Point( x*dx+2*dx,        0 ) ); //go 1 right at upper border
      }
      GridLines.Points[GridLines.Points.Count-1] = new Point(0,0); //go back to upper left corner
      for ( int y=1; y < ySize; y+=2 ) //2 horizontal lines per step
      { GridLines.Points.Add( new Point( 0       , y*dy      ) ); //start at left border
        GridLines.Points.Add( new Point( xSize*dx, y*dy      ) ); //go right to right border
        GridLines.Points.Add( new Point( xSize*dx, y*dy+dy   ) ); //go 1 down at right border
        GridLines.Points.Add( new Point( 0       , y*dy+dy   ) ); //go left to left border
        GridLines.Points.Add( new Point( 0       , y*dy+2*dy ) ); //go 1 down at left border
      }
      //surrounding rectangle
      GridLines.Points[GridLines.Points.Count-1] = new Point(0,0); //upper left  corner
      GridLines.Points.Add( new Point(xSize*dx,       0) );   //goto upper right corner
      GridLines.Points.Add( new Point(xSize*dx,ySize*dy) );   //goto lower right corner
      GridLines.Points.Add( new Point(       0,ySize*dy) );   //goto lower left  corner
      GridLines.Points.Add( new Point(       0,       0) );   //goto upper left  corner
      image0  = new Byte[ySize,xSize];
      image1  = new Byte[ySize,xSize];
      Clear_Click( null, null );
    }
    private void canvas_MouseLeftButtonDown( object sender, MouseButtonEventArgs e ) 
    { Clear_Click( null, null );
      MouseButtonPressed = true;
      DrawSomething.Visibility = Visibility.Collapsed;
    }
    private void canvas_MouseMove( object sender, MouseEventArgs e ) 
    { if ( !MouseButtonPressed ) return;
      Point p = e.GetPosition( DrawCanvas );
      int x = Convert.ToInt32( (p.X - 0.5*dx)/dx ); //if (x==0 | x==xSize-1) return; 
      int y = Convert.ToInt32( (p.Y - 0.5*dy)/dy ); //if (y==0 | y==ySize-1) return; 
      if ( image0[y,x] > 0 ) return; //already set
      image0[y,x] = 9;               //set pixel to white
      drawPixel( y, x, 9 );
    }
    private void canvas_MouseLeftButtonUp( object sender, MouseButtonEventArgs e ) 
    { MouseButtonPressed = false;
    }
    private void drawPixel( int y, int x, int gray )
    { Rectangle r = new Rectangle(); //a rectangle for the pixel
      r.Fill = ShadesOfGray[gray];   //fill it with gray
      r.Width = dx; r.Height = dy;
      Canvas.SetLeft( r, x*dx );     //x-position
      Canvas.SetTop ( r, y*dy );     //y-position
      DrawCanvas.Children.Add( r );  //draw it
    }
    private void Filter_Click( object sender, RoutedEventArgs e ) 
    { DrawCanvas.Children.Clear(); DrawCanvas.Children.Add( GridLines );
      Int32 sum, x, xx, xxx, y, yy, yyy;
      for ( y=1; y < ySize-1; y++ ) //=============
      { for ( x=1; x < xSize-1; x++ ) //===========
        { sum = 0;
          for ( yy=-1; yy <= 1; yy++ ) //==========
          { yyy = y + yy;
            for ( xx=-1; xx <= 1; xx++ )//=========
            { xxx = x + xx;
              sum+=image0[yyy,xxx];
            } //====== end for (int xx... ================
          } //======== end for (int yy... ================
          image1[y,x] = Convert.ToByte( (float)sum/9f );
          drawPixel( y, x, image1[y,x] );
        } //============ end for (int x... =====================
      } //============== end for (int y... =====================
    } // end of private void Filter_Click(...)

    private void Clear_Click( object sender, RoutedEventArgs e ) 
    { for ( int y=0; y < ySize; y++ )
        for ( int x=0; x < xSize; x++ ) image0[y,x] = image1[y,x] = 0;
      DrawCanvas.Children.Clear(); DrawCanvas.Children.Add( GridLines );
                                   DrawCanvas.Children.Add( DrawSomething );
      DrawSomething.Visibility = Visibility.Visible;
      GC.Collect();
    } // end of private void Clear_Click(...)
  } // end of public partial class MainPage:UserControl
} // end of namespace
