﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Shapes;

namespace FastAvFilter5x5
{ public partial class MainPage:UserControl 
  { const Int32 xSize = 11, ySize = 10;
    Double dx, dy;
    int[,] A0, A1, A2; // original-, noisy-, filtered-image
    Rectangle[,] bigPixels; 
    List<SolidColorBrush> ShadesOfGray = new List<SolidColorBrush>();
    public MainPage() 
    { InitializeComponent();
      for ( UInt16 i=0; i <= 255; i++ ) //create 256 brushes
        ShadesOfGray.Add( new SolidColorBrush( Color.FromArgb( 255, (Byte)i, (Byte)i, (Byte)i ) ) );
      A0 = new int[,] { {  0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0},
                        {  0,   0,   0,   0, 200, 200, 200,   0,   0,   0,   0},
                        {  0,   0,   0, 150, 200, 100, 200, 150,   0,   0,   0},
                        {  0,   0,   0,   0, 255, 255, 255,   0,   0,   0,   0},
                        {255,   0,   0,   0,   0, 255,   0,   0,   0,   0, 255},
                        {255, 255, 200, 200, 200, 200, 200, 200, 200, 255, 255},
                        {255,   0,   0,   0, 200, 255, 200,   0,   0,   0, 255},
                        {  0,   0,   0,   0, 200, 200, 200,   0,   0,   0,   0},
                        {  0,   0,   0,   0, 200,   0, 200,   0,   0,   0,   0},
                        {  0,   0,   0,   0, 255,   0, 255,   0,   0,   0,   0} };

      A1 = new int[ySize,xSize];
      A2 = new int[ySize,xSize];
      bigPixels = new Rectangle[ySize,xSize];
    }

    private void canvas_Loaded( object sender, RoutedEventArgs e ) 
    { dx = DrawCanvas.Width / xSize; //pixel width
      dy = DrawCanvas.Height/ ySize; //pixel height
      //Fill the drawing area with rectangles which should represent pixels
      for ( int y=0; y < ySize; y++ )
        for ( int x=0; x < xSize; x++ ) 
        { Rectangle r = bigPixels[y,x] = new Rectangle();
          r.Width  = dx;  
          r.Height = dy;
          Canvas.SetLeft( r, x*dx ); //x-position
          Canvas.SetTop ( r, y*dy ); //y-position
          DrawCanvas.Children.Add( r );
        }
      drawPixels( A0 );
    }

    private void FastAvFilter5x5_Button_Click( object sender, RoutedEventArgs e )
    { int x, y;
      for ( y=0; y < ySize; y++ )     //clear A1 and A2
        for ( x=0; x < xSize; x++ )   A1[y,x] = A2[y,x] = 0;
      for ( y=2; y < ySize-2; y++ )   //horizontal sums
      { for ( x=0; x < 5;       x++ ) A1[y,2] += A0[y,x];
        for ( x=3; x < xSize-2; x++ ) A1[y,x] = A1[y,x-1] - A0[y,x-3] + A0[y,x+2];
      }
      for ( x=2; x < xSize-2; x++ )   //vertical sums
      { for ( y=0; y < 5;       y++ ) A2[2,x] += A1[y,x];
        for ( y=3; y < ySize-2; y++ ) A2[y,x] = A2[y-1,x] - A1[y-3,x] + A1[y+2,x];
      }      
      for ( y=2; y < ySize-2; y++ )   //divide and round
        for ( x=2; x < xSize-2; x++ ) A2[y,x] = Convert.ToInt32( (float)A2[y,x] / 25f );
      borderHandling5x5();
      drawPixels( A2 );
    } // end of private void FastAvFilter5x5()

    private void borderHandling5x5()
    { int x, y;
      for ( y=0; y < 2; y++ )           //upper border
        for ( x=2; x < xSize-2; x++ ) A2[y,x] = A2[2,x]; 
      for ( y=ySize-3; y < ySize; y++ ) //lower border
        for ( x=2; x < xSize-2; x++ ) A2[y,x] = A2[ySize-3,x]; 
      for ( x=0; x < 2; x++ )           //left border
        for ( y=0; y < ySize; y++ )   A2[y,x] = A2[y,2];
      for ( x=xSize-2; x < xSize; x++ ) //right border
        for ( y=0; y < ySize; y++ )   A2[y,x] = A2[y,xSize-3];
    }

    private void Original_Button_Click( object sender, RoutedEventArgs e )
    { drawPixels( A0 );
    }

    private void drawPixels( int[,]  image )
    { for ( int y=0; y < ySize; y++ )
        for ( int x=0; x < xSize; x++ )  
          bigPixels[y,x].Fill = ShadesOfGray[image[y,x]]; //fill them with monchrome gray
    }
  } // end of public partial class MainPage:UserControl
} // end of namespace

