﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Media;
using System.Windows.Shapes;
using System.Globalization;

namespace FastGauss1D 
{ public partial class MainPage:UserControl 
  { const Int32 xSizeMax = 101;
    Int32 hxSizeMax = xSizeMax/2, xSize = 39, hxSize, filterWidth = 3, repetitions = 3;
    Double dx, dy;
    Double   [] row0      ;
    Double   [] row1      ;
    Rectangle[] rectangles;
    List<SolidColorBrush> ShadesOfGray = new List<SolidColorBrush>();
    CultureInfo international = new CultureInfo("en-US");
    SolidColorBrush redBrush   = new SolidColorBrush( Colors.Red   );
    SolidColorBrush greenBrush = new SolidColorBrush( Colors.Green );
    Polyline averStairs = new Polyline();
    Polyline bellStairs = new Polyline();
    double[] sigmas;

    public MainPage() 
    { InitializeComponent();
      for ( UInt16 i=0; i <= 255; i++ ) //create 256 brushes
        ShadesOfGray.Add( new SolidColorBrush( Color.FromArgb( 255, (Byte)i, (Byte)i, (Byte)i ) ) );
      hxSize     = xSize/2; 
      row0       = new Double   [xSizeMax]; row0[hxSize] = 1;
      row1       = new Double   [xSizeMax];
      rectangles = new Rectangle[xSizeMax];
      sigmas     = new double  [hxSizeMax];
      columnsScrollBar    .Maximum = xSizeMax; columnsScrollBar.Value = xSize;
      widthScrollBar      .Maximum = hxSize-1;
      repetitionsScrollBar.Maximum = hxSize/2;
      averStairs.Stroke = redBrush;   averStairs.StrokeThickness=3;  DrawCanvas.Children.Add( averStairs );
      bellStairs.Stroke = greenBrush; bellStairs.StrokeThickness=2;  DrawCanvas.Children.Add( bellStairs );
    }
    private void canvas_Loaded( object sender, RoutedEventArgs e ) 
    { Canvas.SetLeft  ( averStairs, 0 );
      Canvas.SetTop   ( averStairs, 0 );
      Canvas.SetZIndex( averStairs, 4 );
      Canvas.SetLeft  ( bellStairs, 0 );
      Canvas.SetTop   ( bellStairs, 0 );
      Canvas.SetZIndex( bellStairs, 4 );
      for ( int x=0; x < xSizeMax; x++ ) 
      { Rectangle r = rectangles[x] = new Rectangle();
        DrawCanvas.Children.Add( r );
      }
      dx = DrawCanvas.Width / xSize; //rectangle width
      dy = DrawCanvas.Height;        //maximal rectangle height
      for ( int x=0; x < xSize; x++ ) 
      { rectangles[x].Width  = dx;  
        Canvas.SetLeft( rectangles[x], x*dx ); //x-position
      }
      drawAver( row0 );
      widthScrollBar      .Value = filterWidth = 3;
      repetitionsScrollBar.Value = repetitions = 3;
      widthTextBox      .Text = "3";
      repetitionsTextBox.Text = "3";
    }
    private void ScrollBar_ValueChanged(object sender,RoutedPropertyChangedEventArgs<double> e) 
    { try { if ( (ScrollBar)sender == columnsScrollBar ) 
            { xSize = Convert.ToInt32( columnsScrollBar.Value );
              if ( xSize%2 == 0 ) xSize--; //symmetry to left and right 
              columnsTextBox.Text = xSize.ToString();
              averStairs.Points.Clear();
              bellStairs.Points.Clear();
              hxSize = xSize/2;
              widthScrollBar      .Maximum = hxSize-1;
              repetitionsScrollBar.Maximum = hxSize/2;
              widthScrollBar      .Value   = Math.Min( widthScrollBar      .Value, hxSize-1 );
              repetitionsScrollBar.Value   = Math.Min( repetitionsScrollBar.Value, hxSize/2 );
              widthTextBox        .Text    = ((int)widthScrollBar      .Value).ToString();
              repetitionsTextBox  .Text    = ((int)repetitionsScrollBar.Value).ToString();
              for ( int x=0; x < xSize; x++ )
              { row0[x] = row1[x] = 0; 
                dx = DrawCanvas.Width / xSize;
                rectangles[x].Width  = dx;  
                Canvas.SetLeft( rectangles[x], x*dx ); //x-position
              }
              row0[hxSize] = 1;
              drawAver( row0 ); 
            }
            else  if ( (ScrollBar)sender == widthScrollBar ) 
            { filterWidth = Convert.ToInt32( widthScrollBar.Value );
              if ( filterWidth%2 == 0 ) filterWidth--;
              widthTextBox.Text = filterWidth.ToString();
            }
            else                                       
            { repetitions = Convert.ToInt32( repetitionsScrollBar.Value );
              repetitionsTextBox.Text = repetitions.ToString();
            }
          } catch {}
    }
    private void averageFilter()
    { int fwh = filterWidth/2, i, x, xx;
      for ( i=0; i < repetitions; i++ )
      { for ( x=0; x < xSize; x++ ) row1[x] = 0; 
        for ( x=fwh; x < xSize-fwh; x++ )
        { for ( xx=-fwh; xx <= fwh; xx++ ) row1[x] += row0[x+xx];
          row1[x] /= filterWidth;
        }
        for ( x=0; x < xSize; x++ ) row0[x] = row1[x];
      }
    }
    private void FilterButton_Click( object sender, RoutedEventArgs e )
    { for ( int x=0; x < xSize; x++ ) row0[x] = row1[x] = 0;
      row0[hxSize] = 3;
      averageFilter();
      drawAver( row1 );
      drawAverStairs( row1 );
      drawBellStairs( row1 );
    }
    private void drawAver( Double[] row )
    { for ( int x=0; x < xSize; x++ ) 
      { rectangles[x].Height = row[x]*dy;
        Canvas.SetTop ( rectangles[x], (1-row[x])*dy ); //y-position
        int gray = Math.Min( 255, Convert.ToInt32(row[x]*255) );
        rectangles[x].Fill = ShadesOfGray[gray]; 
      }
    }
    private void drawAverStairs( Double[] row )
    { int fwh = filterWidth/2;
      averStairs.Points.Clear();
      for ( int x=0; x < xSize; x++ ) 
        averStairs.Points.Add( new Point( (x+0.5)*dx, (1-row[x])*dy ) );
    }
    private void drawBellStairs( Double[] row )
    { double sigma, sumSigmas = 0;
      int no = 0, fwh = filterWidth/2;
      bellStairs.Points.Clear();
      for ( int x=hxSize-1; x > 0; x-- )
      { if ( row[x] < 0.001 ) row[x] = 0.001;
        if ( row[x] - row[x-1] == 0.001 ) break;
        int d = hxSize - x;
        sigmas[x] = Math.Sqrt( -0.5*d*d / Math.Log( row1[x]/row1[hxSize] ) );
        sumSigmas += sigmas[x];
        no++;
      }
      if ( no > 0 && sumSigmas > 0 )
      { sigma = sumSigmas / no; 
        bellStdDevTextBox.Text = sigma.ToString( "0.0000", international );
        for ( int x=0; x < xSize; x++ ) 
        { int d = hxSize-x;
          double y = row[hxSize]*Math.Exp(-d*d/(2*sigma*sigma));
          Point p = new Point( (x+0.5)*dx, (1-y)*dy );
          bellStairs.Points.Add( p );
        }
      }
      else bellStdDevTextBox.Text = "none";
    }
  }
}
