﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media.Imaging;
using System.Windows.Input;
using System.ComponentModel;
using System.Globalization;

namespace GaussFilterForRealImages
{ public partial class MainPage : UserControl
  { WriteableBitmap img0, img1, kernelImage;
    Int32 kw, hkw;  //kernel width  and half kernel width,  kw uneven and hkw even
    Int32 kh, hkh;  //kernel height and half kernel height, kh uneven and hkh even
    Byte[,] kernel;
    Int32 xSize, ySize; //image width and height
    Double steepness;
    Int64 t0, t1;       //system time before and after filtering
    String fileName;
    Byte [,] R0, G0, B0; //input
    BackgroundWorker myThread = new BackgroundWorker(); //Thread object

    public MainPage()
    { InitializeComponent();
      myThread.DoWork += new DoWorkEventHandler( myThreadDoWork    );
      myThread.RunWorkerCompleted += 
             new RunWorkerCompletedEventHandler( myThreadCompleted );
    }

    private void myImageIn_ImageOpened( object sender,RoutedEventArgs e ) 
    { img0  = new WriteableBitmap( (BitmapSource)myImageIn.Source );
      fileName = ((BitmapImage)myImageIn.Source).UriSource.OriginalString;
      a_new_image_opened();
    }

    private void myButtonOF_Click( object sender, RoutedEventArgs e )
    { OpenFileDialog openFileDialog = new OpenFileDialog();
      openFileDialog.Filter = "Image Files (*.png,*.jpg)|*.png;*.jpg";
      bool? userClickedOK = openFileDialog.ShowDialog();
      if ( userClickedOK == true )
      { try
        { System.IO.Stream stream = openFileDialog.File.OpenRead();
          fileName = openFileDialog.File.ToString();
          BitmapImage bmp = new BitmapImage();
          bmp.SetSource( stream );
          stream.Close();
          img0 = new WriteableBitmap( bmp );
          a_new_image_opened();
        } catch {}
      }
    }

    private void a_new_image_opened()
    { xSize = img0.PixelWidth;
      ySize = img0.PixelHeight;
      img1 = new WriteableBitmap( xSize, ySize );
      myImageIn    .Source = img0;
      myImageOut   .Source = img1;
      //prepare_intermediary_arrays
      R0 = new Byte [ySize, xSize];
      G0 = new Byte [ySize, xSize];
      B0 = new Byte [ySize, xSize];
      GC.Collect(); //clean up memory when new arrays have replaced old ones
      for ( int y=0; y < ySize; y++ )
        for ( int x=0; x < xSize; x++ )
        { Int32 c = img0.Pixels[y*xSize+x];
          R0[y,x] = (Byte)((c & 0xff0000) >> 16);
          G0[y,x] = (Byte)((c & 0x00ff00) >>  8);
          B0[y,x] = (Byte)((c & 0x0000ff)      );
        }
      kw_Slider.Maximum = Math.Min( 61, xSize/4 ); //default 61, small images less
      kh_Slider.Maximum = Math.Min( 61, ySize/4 ); //default 61, small images less
      stdDev_Slider.Value = 0.1; //
      kw_Slider    .Value = 11; //initial kernel width
      kh_Slider    .Value = 11; //initial kernel height
      myTextBoxOF .Text = fileName + ": " + xSize.ToString() + 'x' + ySize.ToString();
      myThreadStart();
    }

    private void myThreadStart()
    { if ( myThread.IsBusy ) return;
      myTextBoxTime.Text = "Wait !";
      myBorder.Cursor = Cursors.Wait;
      double stdDev, steepness;
           if ( stdDev_Slider.Value <= stdDev_Slider.Minimum ) 
      { stdDev = 0.0; steepness = -Single.MaxValue; }
      else 
      { stdDev = Math.Pow( 10, stdDev_Slider.Value );
        steepness = -1.0 / (2.0 * stdDev * stdDev);
      }
      if ( stdDev < 100 ) 
           stdDevTextBox.Text = stdDev.ToString( "0.0", new CultureInfo("en-US") );
      else stdDevTextBox.Text = stdDev.ToString( "0"  , new CultureInfo("en-US") );
      kw = (Int32)kw_Slider.Value; if (kw%2 == 0) kw--; kwTextBox.Text = kw.ToString(); 
      kh = (Int32)kh_Slider.Value; if (kh%2 == 0) kh--; khTextBox.Text = kh.ToString(); 
      hkw = kw / 2;
      hkh = kh / 2;
      //Gaussian bell to kernel
      kernel = new Byte[kh,kw];
      for ( int y=-hkh; y <= hkh; y++ ) //bell to kernel 
        for ( int x=-hkw; x <= hkw; x++ )
        { double vote = 100 * Math.Exp( steepness*(x*x + y*y) );
          kernel[y+hkh,x+hkw ] = Convert.ToByte( vote );
        }
      //visualize kernel
      kernelImage = new WriteableBitmap( kw, kh );
      myImageKernel.Source = kernelImage;
      myImageKernel.Width  = 2*kw;
      myImageKernel.Height = 2*kh;
      for ( int y=0; y < kh; y++ ) 
        for ( int x=0; x < kw; x++ )
        { Byte p = Convert.ToByte( kernel[y,x] * 2.55f );
          unchecked { kernelImage.Pixels[y*kw+x] = 
                      (Int32)0xff000000 | p | (p << 8) | (p << 16); }
        }
      kernelImage.Invalidate();
      kernelTextBox.Text = "lowest vote = " +  kernel[0,0].ToString();    
      t0 = DateTime.Now.Ticks; 
      myThread.RunWorkerAsync(); //Calls myThreadDoWork( ... )
    }

    private void myThreadDoWork( Object sender, DoWorkEventArgs e )
    { GaussFilter ();
    }

    private void myThreadCompleted( Object sender,	RunWorkerCompletedEventArgs e )
    { myBorder.Cursor = Cursors.Arrow;
      t1 = DateTime.Now.Ticks;
      myTextBoxTime.Text = "This filter took " + ((t1-t0)/10000).ToString() + " millisec.";
      img1.Invalidate();
    }

    private void Slider_MouseLeftButtonUp( object sender, MouseButtonEventArgs e ) 
    { myThreadStart();
    }

    private void GaussFilter()
    { int R, G, B, Rsum, Gsum, Bsum, x, xx, xxx, y, yy, yyy, startOfYRow;
      int votes, sumOfVotes;
      for ( y=0; y < ySize; y++ ) //==================
      { startOfYRow = y*xSize;
        for ( x=0; x < xSize; x++ ) //===============
        { Rsum=Gsum=Bsum=sumOfVotes=0;
          for ( yy=-hkh; yy <= hkh; yy++ ) //=============
          { yyy = y + yy;
            if ( yyy < 0 | yyy >= ySize ) continue; //beyond border
            for ( xx=-hkw; xx <= hkw; xx++ )//========
            { xxx = x + xx;
              if ( xxx < 0 | xxx >= xSize ) continue; //beyond border
              sumOfVotes += votes = kernel[yy+hkh,xx+hkw];
              Rsum += R0[yyy,xxx] * votes;
              Gsum += G0[yyy,xxx] * votes;
              Bsum += B0[yyy,xxx] * votes;
            } //====== end for (int xx... ================
          } //======== end for (int yy... ==================
          R=Convert.ToInt32( (float)Rsum/sumOfVotes ) << 16;      
          G=Convert.ToInt32( (float)Gsum/sumOfVotes ) <<  8;
          B=Convert.ToInt32( (float)Bsum/sumOfVotes );
          unchecked { img1.Pixels[startOfYRow+x] = (Int32)0xff000000 | R | G | B; }
        } //============ end for (int x... =====================
      } //============== end for (int y... =======================
    }
  }
}