﻿using System;
using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using System.Windows.Input;
using System.ComponentModel;
using System.Globalization;


namespace MedianFilterOfRealImages
{ public partial class MainPage : UserControl
  { WriteableBitmap img0, img1;
    Int32 kw, hkw;  //kernel width  and half kernel width,  kw uneven and hkw even
    Int32 kh, hkh;  //kernel height and half kernel height, kh uneven and hkh even
    Int32 xSize, ySize; //image width and height
    Int64 t0, t1;       //system time before and after filtering
    String fileName;
    Byte [,] R0, G0, B0; //input
    BackgroundWorker myThread = new BackgroundWorker(); //Thread object

    public MainPage()
    { InitializeComponent();
      myThread.DoWork += new DoWorkEventHandler( myThreadDoWork    );
      myThread.RunWorkerCompleted += 
             new RunWorkerCompletedEventHandler( myThreadCompleted );
    }

    private void myImageIn_ImageOpened( object sender,RoutedEventArgs e ) 
    { img0  = new WriteableBitmap( (BitmapSource)myImageIn.Source );
      fileName = ((BitmapImage)myImageIn.Source).UriSource.OriginalString;
      a_new_image_opened();
    }

    private void myButtonOF_Click( object sender, RoutedEventArgs e )
    { OpenFileDialog openFileDialog = new OpenFileDialog();
      openFileDialog.Filter = "Image Files (*.png,*.jpg)|*.png;*.jpg";
      bool? userClickedOK = openFileDialog.ShowDialog();
      if ( userClickedOK == true )
      { try
        { System.IO.Stream stream = openFileDialog.File.OpenRead();
          fileName = openFileDialog.File.ToString();
          BitmapImage bmp = new BitmapImage();
          bmp.SetSource( stream );
          stream.Close();
          img0 = new WriteableBitmap( bmp );
          a_new_image_opened();
        } catch {}
      }
    }

    private void a_new_image_opened()
    { xSize = img0.PixelWidth;
      ySize = img0.PixelHeight;
      img1 = new WriteableBitmap( xSize, ySize );
      myImageIn    .Source = img0;
      myImageOut   .Source = img1;
      //prepare_intermediary_arrays
      R0 = new Byte [ySize, xSize];
      G0 = new Byte [ySize, xSize];
      B0 = new Byte [ySize, xSize];
      GC.Collect(); //clean up memory when new arrays have replaced old ones
      for ( int y=0; y < ySize; y++ )
        for ( int x=0; x < xSize; x++ )
        { Int32 c = img0.Pixels[y*xSize+x];
          R0[y,x] = (Byte)((c & 0xff0000) >> 16);
          G0[y,x] = (Byte)((c & 0x00ff00) >>  8);
          B0[y,x] = (Byte)((c & 0x0000ff)      );
        }
      kw_Slider.Maximum = Math.Min( 61, xSize/4 ); //default 61, small images less
      kh_Slider.Maximum = Math.Min( 61, ySize/4 ); //default 61, small images less
      kw_Slider   .Value = 7; //initial kernel width
      kh_Slider   .Value = 7; //initial kernel height
      myTextBoxOF .Text = fileName + ": " + xSize.ToString() + 'x' + ySize.ToString();
      myThreadStart();
    }

    private void myThreadStart()
    { if ( myThread.IsBusy ) return;
      myTextBoxTime.Text = "Wait !";
      myBorder.Cursor = Cursors.Wait;
      kw = (Int32)kw_Slider.Value; if (kw%2 == 0) kw--; kwTextBox.Text = kw.ToString(); 
      kh = (Int32)kh_Slider.Value; if (kh%2 == 0) kh--; khTextBox.Text = kh.ToString(); 
      hkw = kw / 2;
      hkh = kh / 2;
      t0 = DateTime.Now.Ticks; 
      myThread.RunWorkerAsync(); //Calls myThreadDoWork( ... )
    }

    private void myThreadDoWork( Object sender, DoWorkEventArgs e )
    { MedianFilter ();
    }

    private void myThreadCompleted( Object sender,	RunWorkerCompletedEventArgs e )
    { myBorder.Cursor = Cursors.Arrow;
      t1 = DateTime.Now.Ticks;
      myTextBoxTime.Text = "This filter took " + ((t1-t0)/10000).ToString() + " millisec.";
      img1.Invalidate();
    }

    private void Slider_MouseLeftButtonUp( object sender, MouseButtonEventArgs e ) 
    { myThreadStart();
    }

    private void MedianFilter()
    { int R, G, B, no, noh, x, xx, xxx, y, yy, yyy, startOfYRow;
      Byte[] RR = new Byte[kw*kh];
      Byte[] GG = new Byte[kw*kh];
      Byte[] BB = new Byte[kw*kh];
      for ( y=0; y < ySize; y++ ) //==================
      { startOfYRow = y*xSize;
        for ( x=0; x < xSize; x++ ) //===============
        { no=0;
          for ( yy=-hkh; yy <= hkh; yy++ ) //=============
          { yyy = y + yy;
            if ( yyy < 0 | yyy >= ySize ) continue; //beyond border
            for ( xx=-hkw; xx <= hkw; xx++ )//========
            { xxx = x + xx;
              if ( xxx < 0 | xxx >= xSize ) continue; //beyond border
              RR[no] = R0[yyy,xxx];
              GG[no] = G0[yyy,xxx];
              BB[no] = B0[yyy,xxx];
              no++;
            } //====== end for (int xx... ================
          } //======== end for (int yy... ================
          Array.Sort( RR, 0, no );
          Array.Sort( GG, 0, no );
          Array.Sort( BB, 0, no );
          noh = no/2;
          R=RR[noh];      
          G=GG[noh];
          B=BB[noh];
          unchecked { img1.Pixels[startOfYRow+x] = (Int32)0xff000000 | (R << 16) | (G << 8) | B; }
        } //============ end for (int x... =====================
      } //============== end for (int y... =====================
    }
  }
}