﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using System.Windows.Input;
using System.ComponentModel;

namespace DCT2DColor
{ public partial class MainPage : UserControl
  { WriteableBitmap img0, img1;
    Int32 xSize, ySize; //image width+height
    Int32 backHorizontalLow=10, backHorizontalHigh, backVerticalLow=10, backVerticalHigh; //limits of the back transform
    Int64 t0, t1;       //system time before and after filtering
    String fileName; 
    Byte [,] R0, G0, B0;                         //input arrays for red, green, blue
    float[,] R1, G1, B1, R2, G2, B2, R3, G3, B3; //DCT   arrays for red, green, blue
    BackgroundWorker myThread = new BackgroundWorker(); //Thread object
    bool new_image_opened_flag = true, BackVerticalSliderMoved = true;

    public MainPage()
    { InitializeComponent();
      myThread.DoWork += new DoWorkEventHandler( thread_DoWork );
      myThread.RunWorkerCompleted += 
        new RunWorkerCompletedEventHandler( myThreadCompleted );
    }

    private void myImageIn_ImageOpened( object sender,RoutedEventArgs e ) 
    { img0  = new WriteableBitmap( (BitmapSource)myImageIn.Source );
      fileName = ((BitmapImage)myImageIn.Source).UriSource.OriginalString;
      a_new_image_opened();
    }

    private void myButtonOF_Click( object sender, RoutedEventArgs e )
    { OpenFileDialog openFileDialog = new OpenFileDialog();
      openFileDialog.Filter = "Image Files (*.png,*.jpg)|*.png;*.jpg";
      bool? userClickedOK = openFileDialog.ShowDialog();
      if ( userClickedOK == true )
      { try
        { System.IO.Stream stream = openFileDialog.File.OpenRead();
          fileName = openFileDialog.File.ToString();
          BitmapImage bmp = new BitmapImage();
          bmp.SetSource( stream );
          stream.Close();
          img0 = new WriteableBitmap( bmp );
          a_new_image_opened();
          new_image_opened_flag = true;
          BackVerticalSliderMoved   = true;
        } catch {}
      }
    }

    private void a_new_image_opened()
    { xSize = img0.PixelWidth;
      ySize = img0.PixelHeight;
      img1 = new WriteableBitmap( xSize, ySize );
      myImageIn .Source = img0;
      myImageOut.Source = img1;
      myTextBlockOF.Text = fileName + ": " + xSize.ToString() + 'x' + ySize.ToString();
      if ( xSize*ySize > 262144 )
      { myTextBlockTime.Text = "This image is too big to be transformed in reasonable time!";
        BHLTextBox.Text=BHHTextBox.Text=BVLTextBox.Text=BVHTextBox.Text = "";
        BHLSlider.Visibility = BHHSlider.Visibility =
        BVLSlider.Visibility = BVHSlider.Visibility = Visibility.Collapsed;
        return;
      }
      BHLSlider.Visibility = BHHSlider.Visibility =
      BVLSlider.Visibility = BVHSlider.Visibility = Visibility.Visible;
      //prepare_intermediary_arrays
      R0  = new Byte [ySize,xSize]; //spatial domain input red
      G0  = new Byte [ySize,xSize]; //spatial domain input green
      B0  = new Byte [ySize,xSize]; //spatial domain input blue
      R1  = new float[ySize,xSize]; //horizontal DCT red
      G1  = new float[ySize,xSize]; //horizontal DCT green
      B1  = new float[ySize,xSize]; //horizontal DCT blue
      R2  = new float[ySize,xSize]; //full DCT red
      G2  = new float[ySize,xSize]; //full DCT green
      B2  = new float[ySize,xSize]; //full DCT blue
      R3  = new float[ySize,xSize]; //horizontal DCT back red
      G3  = new float[ySize,xSize]; //horizontal DCT back green
      B3  = new float[ySize,xSize]; //horizontal DCT back blue
      
      GC.Collect(); //clean up memory
      for ( int y=0; y < ySize; y++ )
        for ( int x=0; x < xSize; x++ )
        { Int32 p = img0.Pixels[y*xSize+x];
          R0[y,x] = (Byte)(( p & 0xff0000) >> 16);
          G0[y,x] = (Byte)(( p & 0x00ff00) >>  8);
          B0[y,x] = (Byte)(( p & 0x0000ff)      );
        }
      BHHSlider.Maximum  = BHLSlider.Maximum=BHHSlider.Value=backHorizontalHigh=xSize-1;
      BVHSlider.Maximum  = BVLSlider.Maximum=BVHSlider.Value=backVerticalHigh  =ySize-1;
      BHLSlider.Value    = BVLSlider.Value = backHorizontalLow = backVerticalLow = 1;
      BHLTextBox.Text    = BVLTextBox.Text = "1";
      BHHTextBox.Text    = backHorizontalHigh.ToString();
      BVHTextBox.Text    = backVerticalHigh  .ToString();
      myThreadStart();
    }

    private void myThreadStart()
    { if ( myThread.IsBusy ) return;
      myTextBlockTime.Text = "Wait !";
      myBorder.Cursor = Cursors.Wait;
      t0 = DateTime.Now.Ticks; 
      myThread.RunWorkerAsync();
    }

    private void thread_DoWork( Object sender, DoWorkEventArgs e )
    { if ( new_image_opened_flag == true )  
      { DCT1D_HorizontallyAllRows(); 
        DCT1D_VerticallyAllColumns();
      }
      if ( BackVerticalSliderMoved == true ) 
        InverseDCT1D_VerticallyAllColumns( backVerticalLow, backVerticalHigh );
      InverseDCT1D_HorizontallyAllRows( backHorizontalLow, backHorizontalHigh );
    }

    private void myThreadCompleted( Object sender,	RunWorkerCompletedEventArgs e )
    { myBorder.Cursor = Cursors.Arrow;
      t1 = DateTime.Now.Ticks;
      myTextBlockTime.Text = "This filter took " + ((t1-t0)/10000).ToString() + " millisec.";
      img1.Invalidate();
    }

    private void Slider_MouseLeftButtonUp( object sender, MouseButtonEventArgs e ) 
    { switch ( ((Slider)sender).Name )
      { case "BHLSlider": 
          backHorizontalLow = (Int32)BHLSlider.Value;
          if ( backHorizontalLow > xSize - 3 ) backHorizontalLow = xSize - 3;
          if ( backHorizontalHigh - backHorizontalLow < 2 ) backHorizontalHigh = backHorizontalLow+2;
          BHLTextBox.Text = backHorizontalLow .ToString();
          BHHTextBox.Text = backHorizontalHigh.ToString();
          BHHSlider.Value = backHorizontalHigh;
          BackVerticalSliderMoved = false;
          break;
        case "BHHSlider": 
          backHorizontalHigh = (Int32)BHHSlider.Value;
          if ( backHorizontalHigh < 3 ) backHorizontalHigh = 3;
          if ( backHorizontalHigh - backHorizontalLow < 2 ) backHorizontalLow = backHorizontalHigh-2;
          BHLTextBox.Text = backHorizontalLow .ToString();
          BHHTextBox.Text = backHorizontalHigh.ToString();
          BHLSlider.Value = backHorizontalLow;
         BackVerticalSliderMoved = false;
          break;
        case "BVLSlider": 
          backVerticalLow = (Int32)BVLSlider.Value;
          if ( backVerticalLow > ySize-3 ) backVerticalLow = ySize - 3;
          if ( backVerticalHigh - backVerticalLow < 2 ) backVerticalHigh = backVerticalLow+2;
          BVLTextBox.Text = backVerticalLow .ToString();
          BVHTextBox.Text = backVerticalHigh.ToString();
          BVHSlider.Value = backVerticalHigh;
          BackVerticalSliderMoved = true;
          break;
        case "BVHSlider": 
          backVerticalHigh = (Int32)BVHSlider.Value;
          if ( backVerticalHigh < 3 ) backVerticalHigh = 3;
          if ( backVerticalHigh - backVerticalLow < 2 ) backVerticalLow = backVerticalHigh-2;
          BVLTextBox.Text = backVerticalLow .ToString();
          BVHTextBox.Text = backVerticalHigh.ToString();
          BVLSlider.Value = backVerticalLow;
          BackVerticalSliderMoved = true;
          break;
      }
      new_image_opened_flag = false;
      myThreadStart();
    }

    private void DCT1D_HorizontallyAllRows() //horizontal DCT
    { int x, xx, y;
      float sumR, sumG, sumB, cosinus;
      float scaleFactor0 = (float)( Math.Sqrt( 1.0/xSize ) );
      float scaleFactor  = (float)( Math.Sqrt( 2.0/xSize ) );
      double piN =   Math.PI / xSize;
      for ( y=0; y < ySize; y++ )   //for all rows
        for ( x=0; x < xSize; x++ ) //DCT1D horizontal
        { sumR = sumG = sumB = 0;
          for ( xx=0; xx < xSize; xx++ ) 
          { cosinus = (float)Math.Cos( piN*(xx+0.5)*x );
            sumR += R0[y,xx] * cosinus;
            sumG += G0[y,xx] * cosinus;
            sumB += B0[y,xx] * cosinus;
          }
          if ( x == 0 ) { R1[y,x] = sumR*scaleFactor0;
                          G1[y,x] = sumG*scaleFactor0;
                          B1[y,x] = sumB*scaleFactor0;
                        }
          else          { R1[y,x] = sumR*scaleFactor;
                          G1[y,x] = sumG*scaleFactor;
                          B1[y,x] = sumB*scaleFactor;
                        }
        }//end of for ( x=0; x < xSize; x++ ) //DCT1D horizontal
    }
    private void DCT1D_VerticallyAllColumns() //vertical DCT
    { int x, y, yy;
      float sumR, sumG, sumB, cosinus;
      float scaleFactor0 = (float)( Math.Sqrt( 1.0/xSize ) );
      float scaleFactor  = (float)( Math.Sqrt( 2.0/xSize ) );
      double piN =   Math.PI / ySize;
      for ( x=0; x < xSize; x++ )   //for all columns
        for ( y=0; y < ySize; y++ ) //DCT1D vertical
        { sumR = sumG = sumB = 0;
          for ( yy=0; yy < ySize; yy++ ) 
          { cosinus = (float)Math.Cos( piN*(yy+0.5)*y );
            sumR += R1[yy,x] * cosinus;
            sumG += G1[yy,x] * cosinus;
            sumB += B1[yy,x] * cosinus;
          }
          if ( y == 0 ) { R2[y,x] = sumR*scaleFactor0;
                          G2[y,x] = sumG*scaleFactor0;
                          B2[y,x] = sumB*scaleFactor0;
                        }
          else          { R2[y,x] = sumR*scaleFactor;
                          G2[y,x] = sumG*scaleFactor;
                          B2[y,x] = sumB*scaleFactor;
                        }
        }//end of for ( y=0; y < ySize; y++ ) //DCT1D vertical
    }
    private void InverseDCT1D_VerticallyAllColumns( int low, int high ) //vertical back
    { int x, y, yy;
      float sumR, sumG, sumB, cosinus;
      float scaleFactor0 = (float)( Math.Sqrt( 1.0/xSize ) );
      float scaleFactor  = (float)( Math.Sqrt( 2.0/xSize ) );
      double piN =   Math.PI / ySize;
      for ( x=0; x < xSize; x++ )   //for all columns
        for ( y=0; y < ySize; y++ ) //inverse DCT1D vertical 
        { sumR = R2[0,x] * scaleFactor0;
          sumG = G2[0,x] * scaleFactor0;
          sumB = B2[0,x] * scaleFactor0;
          for ( yy=low; yy < high; yy++ )
          { cosinus = scaleFactor * (float)Math.Cos( piN*yy*(y+0.5) );
            sumR += R2[yy,x] * cosinus;
            sumG += G2[yy,x] * cosinus;
            sumB += B2[yy,x] * cosinus;
          }
          R3[y,x] = sumR;
          G3[y,x] = sumG;
          B3[y,x] = sumB;
        }//end of for ( y=0; y < ySize; y++ ) //inverse DCT1D vertical 
    }
    private void InverseDCT1D_HorizontallyAllRows( int low, int heigh ) //horizontal back
    { int x, xx, y, R, G, B;
      float sumR, sumG, sumB, cosinus;
      float scaleFactor0 = (float)( Math.Sqrt( 1.0/xSize ) );
      float scaleFactor  = (float)( Math.Sqrt( 2.0/xSize ) );
      double piN =   Math.PI / xSize;
      for ( y=0; y < ySize; y++ )   //for all rows
        for ( x=0; x < xSize; x++ ) //inverse DCT1D horizontal 
        { sumR = R3[y,0] * scaleFactor0;
          sumG = G3[y,0] * scaleFactor0;
          sumB = B3[y,0] * scaleFactor0;
          for ( xx=low; xx < heigh; xx++ )
          { cosinus = scaleFactor * (float)Math.Cos( piN*xx*(x+0.5) );
            sumR += R3[y,xx] * cosinus;
            sumG += G3[y,xx] * cosinus;
            sumB += B3[y,xx] * cosinus;
          }
          R = Convert.ToInt32( sumR ); if ( R < 0 ) R = 0; if ( R > 255 ) R = 255;
          G = Convert.ToInt32( sumG ); if ( G < 0 ) G = 0; if ( G > 255 ) G = 255;
          B = Convert.ToInt32( sumB ); if ( B < 0 ) B = 0; if ( B > 255 ) B = 255;
          unchecked { img1.Pixels[y*xSize+x] = 
                      (Int32)0xff000000 | (R << 16) | (G << 8) | B; }
       }//end of for ( x=0; x < xSize; x++ ) //inverse DCT1D horizontal 
    }//end of private void InverseDCT1D_HorizontallyAllRows(...)
  }//end of class MainPage
}//end of namespace DCT2DColor