﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using System.Windows.Input;
using System.ComponentModel;

namespace DCT2DMonochrome
{ public partial class MainPage : UserControl
  { WriteableBitmap img0, img1;
    Int32 xSize, ySize; //image width+height
    Int32 backHorizontalLow=10, backHorizontalHigh, backVerticalLow=10, backVerticalHigh; //limits of the back transform
    Int64 t0, t1;       //system time before and after filtering
    String fileName; 
    Byte [,] R0;          //input  array
    float[,] R1, R2, R3; //DCT    arrays
    BackgroundWorker myThread = new BackgroundWorker(); //Thread object
    bool new_image_opened_flag = true, BackVerticalSliderMoved = true;

    public MainPage()
    { InitializeComponent();
      myThread.DoWork += new DoWorkEventHandler( thread_DoWork );
      myThread.RunWorkerCompleted += 
        new RunWorkerCompletedEventHandler( myThreadCompleted );
    }

    private void myImageIn_ImageOpened( object sender,RoutedEventArgs e ) 
    { img0  = new WriteableBitmap( (BitmapSource)myImageIn.Source );
      fileName = ((BitmapImage)myImageIn.Source).UriSource.OriginalString;
      a_new_image_opened();
    }

    private void myButtonOF_Click( object sender, RoutedEventArgs e )
    { OpenFileDialog openFileDialog = new OpenFileDialog();
      openFileDialog.Filter = "Image Files (*.png,*.jpg)|*.png;*.jpg";
      bool? userClickedOK = openFileDialog.ShowDialog();
      if ( userClickedOK == true )
      { try
        { System.IO.Stream stream = openFileDialog.File.OpenRead();
          fileName = openFileDialog.File.ToString();
          BitmapImage bmp = new BitmapImage();
          bmp.SetSource( stream );
          stream.Close();
          img0 = new WriteableBitmap( bmp );
          a_new_image_opened();
          new_image_opened_flag = true;
          BackVerticalSliderMoved   = true;
        } catch {}
      }
    }

    private void a_new_image_opened()
    { xSize = img0.PixelWidth;
      ySize = img0.PixelHeight;
      img1 = new WriteableBitmap( xSize, ySize );
      myImageIn .Source = img0;
      myImageOut.Source = img1;
      myTextBlockOF.Text = fileName + ": " + xSize.ToString() + 'x' + ySize.ToString();
      if ( xSize*ySize > 262144 )
      { myTextBlockTime.Text = "This image is too big to be transformed in reasonable time!";
        BHLTextBox.Text=BHHTextBox.Text=BVLTextBox.Text=BVHTextBox.Text = "";
        BHLSlider.Visibility = BHHSlider.Visibility =
        BVLSlider.Visibility = BVHSlider.Visibility = Visibility.Collapsed;
        return;
      }
      BHLSlider.Visibility = BHHSlider.Visibility =
      BVLSlider.Visibility = BVHSlider.Visibility = Visibility.Visible;
      //prepare_intermediary_arrays
      R0  = new Byte [ySize,xSize]; //spatial domain input
      R1  = new float[ySize,xSize]; //horizontal DCT 
      R2  = new float[ySize,xSize]; //full DCT
      R3  = new float[ySize,xSize]; //horizontal DCT back
      
      GC.Collect(); //clean up memory
      for ( int y=0; y < ySize; y++ )
        for ( int x=0; x < xSize; x++ )
        { Int32 p = img0.Pixels[y*xSize+x];
          R0[y,x] = (Byte)((p & 0xff0000) >> 16);
        }
      BHHSlider.Maximum  = BHLSlider.Maximum=BHHSlider.Value=backHorizontalHigh=xSize-1;
      BVHSlider.Maximum  = BVLSlider.Maximum=BVHSlider.Value=backVerticalHigh  =ySize-1;
      BHLSlider.Value    = BVLSlider.Value = backHorizontalLow = backVerticalLow = 1;
      BHLTextBox.Text    = BVLTextBox.Text = "1";
      BHHTextBox.Text    = backHorizontalHigh.ToString();
      BVHTextBox.Text    = backVerticalHigh  .ToString();
      myThreadStart();
    }

    private void myThreadStart()
    { if ( myThread.IsBusy ) return;
      myTextBlockTime.Text = "Wait !";
      myBorder.Cursor = Cursors.Wait;
      t0 = DateTime.Now.Ticks; 
      myThread.RunWorkerAsync();
    }

    private void thread_DoWork( Object sender, DoWorkEventArgs e )
    { if ( new_image_opened_flag == true )  
      { DCT1D_HorizontallyAllRows(); 
        DCT1D_VerticallyAllColumns();
      }
      if ( BackVerticalSliderMoved == true ) 
        InverseDCT1D_VerticallyAllColumns( backVerticalLow, backVerticalHigh );
      InverseDCT1D_HorizontallyAllRows( backHorizontalLow, backHorizontalHigh );
    }

    private void myThreadCompleted( Object sender,	RunWorkerCompletedEventArgs e )
    { myBorder.Cursor = Cursors.Arrow;
      t1 = DateTime.Now.Ticks;
      myTextBlockTime.Text = "This filter took " + ((t1-t0)/10000).ToString() + " millisec.";
      img1.Invalidate();
    }

    private void Slider_MouseLeftButtonUp( object sender, MouseButtonEventArgs e ) 
    { switch ( ((Slider)sender).Name )
      { case "BHLSlider": 
          backHorizontalLow = (Int32)BHLSlider.Value;
          if ( backHorizontalLow > xSize - 3 ) backHorizontalLow = xSize - 3;
          if ( backHorizontalHigh - backHorizontalLow < 2 ) backHorizontalHigh = backHorizontalLow+2;
          BHLTextBox.Text = backHorizontalLow .ToString();
          BHHTextBox.Text = backHorizontalHigh.ToString();
          BHHSlider.Value = backHorizontalHigh;
          BackVerticalSliderMoved = false;
          break;
        case "BHHSlider": 
          backHorizontalHigh = (Int32)BHHSlider.Value;
          if ( backHorizontalHigh < 3 ) backHorizontalHigh = 3;
          if ( backHorizontalHigh - backHorizontalLow < 2 ) backHorizontalLow = backHorizontalHigh-2;
          BHLTextBox.Text = backHorizontalLow .ToString();
          BHHTextBox.Text = backHorizontalHigh.ToString();
          BHLSlider.Value = backHorizontalLow;
         BackVerticalSliderMoved = false;
          break;
        case "BVLSlider": 
          backVerticalLow = (Int32)BVLSlider.Value;
          if ( backVerticalLow > ySize-3 ) backVerticalLow = ySize - 3;
          if ( backVerticalHigh - backVerticalLow < 2 ) backVerticalHigh = backVerticalLow+2;
          BVLTextBox.Text = backVerticalLow .ToString();
          BVHTextBox.Text = backVerticalHigh.ToString();
          BVHSlider.Value = backVerticalHigh;
          BackVerticalSliderMoved = true;
          break;
        case "BVHSlider": 
          backVerticalHigh = (Int32)BVHSlider.Value;
          if ( backVerticalHigh < 3 ) backVerticalHigh = 3;
          if ( backVerticalHigh - backVerticalLow < 2 ) backVerticalLow = backVerticalHigh-2;
          BVLTextBox.Text = backVerticalLow .ToString();
          BVHTextBox.Text = backVerticalHigh.ToString();
          BVLSlider.Value = backVerticalLow;
          BackVerticalSliderMoved = true;
          break;
      }
      new_image_opened_flag = false;
      myThreadStart();
    }

    private void DCT1D_HorizontallyAllRows() //horizontal DCT
    { int x, xx, y;
      float sum;
      float scaleFactor0 = (float)( Math.Sqrt( 1.0/xSize ) );
      float scaleFactor  = (float)( Math.Sqrt( 2.0/xSize ) );
      double piN =   Math.PI / xSize;
      for ( y=0; y < ySize; y++ )   //for all rows
        for ( x=0; x < xSize; x++ ) //DCT1D horizontal
        { sum = 0;
          for ( xx=0; xx < xSize; xx++ ) sum += R0[y,xx] * (float)Math.Cos( piN*(xx+0.5)*x );
          if ( x == 0 ) R1[y,x] = sum*scaleFactor0;
          else          R1[y,x] = sum*scaleFactor;
        }
     }
    private void DCT1D_VerticallyAllColumns() //vertical DCT
    { int x, y, yy;
      float sum;
      float scaleFactor0 = (float)( Math.Sqrt( 1.0/xSize ) );
      float scaleFactor  = (float)( Math.Sqrt( 2.0/xSize ) );
      double piN =   Math.PI / ySize;
      for ( x=0; x < xSize; x++ )   //for all columns
        for ( y=0; y < ySize; y++ ) //DCT1D vertical
        { sum = 0;
          for ( yy=0; yy < ySize; yy++ ) sum += R1[yy,x] * (float)Math.Cos( piN*(yy+0.5)*y );
          if ( y == 0 ) R2[y,x] = sum*scaleFactor0;
          else          R2[y,x] = sum*scaleFactor;
        }
     }
    private void InverseDCT1D_VerticallyAllColumns( int low, int high ) //vertical back
    { int x, y, yy;
      float sum;
      float scaleFactor0 = (float)( Math.Sqrt( 1.0/xSize ) );
      float scaleFactor  = (float)( Math.Sqrt( 2.0/xSize ) );
      double piN =   Math.PI / ySize;
      for ( x=0; x < xSize; x++ )   //for all columns
        for ( y=0; y < ySize; y++ ) //inverse DCT1D vertical 
        { sum = R2[0,x] * scaleFactor0;
          for ( yy=low; yy < high; yy++ ) 
            sum += R2[yy,x] * scaleFactor * (float)Math.Cos( piN*yy*(y+0.5) );
          R3[y,x] = sum;
        }
    }
    private void InverseDCT1D_HorizontallyAllRows( int low, int heigh ) //horizontal back
    { int x, xx, y, pixel;
      float sum;
      float scaleFactor0 = (float)( Math.Sqrt( 1.0/xSize ) );
      float scaleFactor  = (float)( Math.Sqrt( 2.0/xSize ) );
      double piN =   Math.PI / xSize;
      for ( y=0; y < ySize; y++ )   //for all rows
        for ( x=0; x < xSize; x++ ) //inverse DCT1D horizontal 
        { sum = R3[y,0] * scaleFactor0;
          for ( xx=low; xx < heigh; xx++ )
            sum += R3[y,xx] * scaleFactor * (float)Math.Cos( piN*xx*(x+0.5) );
          pixel = Convert.ToInt32( sum ); 
          if ( pixel < 0 ) pixel = 0; if ( pixel > 255 ) pixel = 255;
          unchecked { img1.Pixels[y*xSize+x] = 
                      (Int32)0xff000000 | (pixel << 16) | (pixel << 8) | pixel; }
       }
    }
  }
}