﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media.Imaging;
using System.Windows.Input;
using System.ComponentModel;

namespace LaplaceFilterForRealImages
{ public partial class MainPage : UserControl
  { WriteableBitmap img0, imLMR, imRML, imUML, imLMU, imAll;
    float ContrastFactor = 1f; //contrast enhancement factor
    Int32 xSize, ySize; //image width and height
    Int64 t0, t1;       //system time before and after filtering
    String fileName; 
    Byte [,] R0, G0, B0; //intermediary
    BackgroundWorker myThread = new BackgroundWorker(); //Thread object

    public MainPage()
    { InitializeComponent();
      myThread.DoWork += new DoWorkEventHandler( thread_DoWork );
      myThread.RunWorkerCompleted += 
        new RunWorkerCompletedEventHandler( update_View_when_thread_completed );
      myThread.WorkerSupportsCancellation = true;
    }

    private void myImageIn_ImageOpened( object sender,RoutedEventArgs e ) 
    { img0  = new WriteableBitmap( (BitmapSource)myImageIn.Source );
      fileName = ((BitmapImage)myImageIn.Source).UriSource.OriginalString;
      a_new_image_opened();
    }

    private void myButtonOF_Click( object sender, RoutedEventArgs e )
    { OpenFileDialog openFileDialog = new OpenFileDialog();
      openFileDialog.Filter = "Image Files (*.png,*.jpg)|*.png;*.jpg";
      bool? userClickedOK = openFileDialog.ShowDialog();
      if ( userClickedOK == true )
      { try
        { System.IO.Stream stream = openFileDialog.File.OpenRead();
          fileName = openFileDialog.File.ToString();
          BitmapImage bmp = new BitmapImage();
          bmp.SetSource( stream );
          stream.Close();
          img0 = new WriteableBitmap( bmp );
          a_new_image_opened();
        } catch {}
      }
    }

    private void a_new_image_opened()
    { xSize = img0.PixelWidth;
      ySize = img0.PixelHeight;
      imLMR = new WriteableBitmap( xSize, ySize );
      imRML = new WriteableBitmap( xSize, ySize );
      imUML = new WriteableBitmap( xSize, ySize );
      imLMU = new WriteableBitmap( xSize, ySize );
      imAll = new WriteableBitmap( xSize, ySize );
      myImageIn .Source = img0;
      myImageOut.Source = imAll;
      //prepare_intermediary_arrays
      R0 = new Byte [ySize, xSize];
      G0 = new Byte [ySize, xSize];
      B0 = new Byte [ySize, xSize];
      GC.Collect(); //clean up memory when new arrays override old ones
      for ( int y=0; y < ySize; y++ )
        for ( int x=0; x < xSize; x++ )
        { Int32 c = img0.Pixels[y*xSize+x];
          R0[y,x] = (Byte)((c & 0xff0000) >> 16);
          G0[y,x] = (Byte)((c & 0x00ff00) >>  8);
          B0[y,x] = (Byte) (c & 0x0000ff)       ;
        }
      if ( update_View_before_starting_thread() ) start_myThread();
    }

    private bool update_View_before_starting_thread()
    { myTextBlockOF.Text = fileName + ": " +
          xSize.ToString() + 'x' + ySize.ToString();
      myTextBlockTime.Text = "Wait !"; return true;
    }

    private void start_myThread()
    { if ( myThread.IsBusy ) return;
      myBorder.Cursor = Cursors.Wait;
      t0 = DateTime.Now.Ticks; 
      myThread.RunWorkerAsync();
    }

    private void update_View_when_thread_completed( Object sender,
                                                    RunWorkerCompletedEventArgs e )
    { myImageOut.Source = imAll;
      myBorder.Cursor = Cursors.Arrow;
      if ( e.Cancelled ) 
      { myTextBlockTime.Text = "";
      }
      else 
      { t1 = DateTime.Now.Ticks;
        myTextBlockTime.Text =  ((t1-t0)/10000).ToString() + " millisec.";
      }
    }

    private void thread_DoWork( Object sender, DoWorkEventArgs e )
    { LaplaceFilter( e );
    }

    /* Common event handler of 5 Buttons of the GUI *****************************/
    private void filter_Click( object sender, RoutedEventArgs e )
    { switch ( ( (Button)sender).Content.ToString() )
      { case "RightMinusLeft" : myImageOut.Source = imRML; break;
        case "LeftMinusRight" : myImageOut.Source = imLMR; break;
        case "LowerMinusUpper": myImageOut.Source = imLMU; break;
        case "UpperMinusLower": myImageOut.Source = imUML; break;
        case "All"            : myImageOut.Source = imAll; break;
      }
    }

    /* The global value "ContrastFactor" is set by this slider of the GUI ********/
    private void Slider_MouseLeftButtonUp( object sender, MouseButtonEventArgs e ) 
    { ContrastFactor = (float)ContrastFactorSlider.Value; //contrast factor
      ContrastFactorTextBox.Text = ContrastFactor.ToString("#.#");
      if ( update_View_before_starting_thread() ) start_myThread();
    }

/******** Here begins the filter algorithm. **************************************/
/******** It would be more professional to write it into a separate class file. **/ 
   
    /* This function "diff" is called 12 times by "LaplaceFilter". ***************/
    /* The global value "ContrastFactor" is set by a slider of the GUI ***********/
    private int diff( int d ) //enhance difference and clip the result to 0..255
    { if ( d <= 0 ) return 0; //clip negative values to 0
      return Convert.ToInt32(Math.Min(255f,  ContrastFactor*d));
    }

    private void LaplaceFilter( DoWorkEventArgs e )
    { int R, G, B, Rmax, Gmax, Bmax, pointer; 
      for ( int y=0; y < ySize-1; y++ )
      { if ( myThread.CancellationPending ) e.Cancel = true; //kill the thread
        pointer = y*xSize; //points now to the first pixel of row y
        for ( int x=0; x < xSize-1; x++, pointer++ )
        { R = diff( R0[y,x+1] - R0[y,x] ); Rmax = R;
          G = diff( G0[y,x+1] - G0[y,x] ); Gmax = G;
          B = diff( B0[y,x+1] - B0[y,x] ); Bmax = B;
          unchecked { imRML.Pixels[pointer] = (Int32)0xff000000 | R << 16 | G << 8 | B; }
          R = diff( R0[y,x] - R0[y,x+1] ); Rmax = Math.Max(R, Rmax);
          G = diff( G0[y,x] - G0[y,x+1] ); Gmax = Math.Max(G, Gmax);
          B = diff( B0[y,x] - B0[y,x+1] ); Bmax = Math.Max(B, Bmax);
          unchecked { imLMR.Pixels[pointer] = (Int32)0xff000000 | R << 16 | G << 8 | B; }
          R = diff( R0[y+1,x] - R0[y,x] ); Rmax = Math.Max(R, Rmax);
          G = diff( G0[y+1,x] - G0[y,x] ); Gmax = Math.Max(G, Gmax);
          B = diff( B0[y+1,x] - B0[y,x] ); Bmax = Math.Max(B, Bmax);
          unchecked { imLMU.Pixels[pointer] = (Int32)0xff000000 | R << 16 | G << 8 | B; }
          R = diff( R0[y,x] - R0[y+1,x] ); Rmax = Math.Max(R, Rmax);
          G = diff( G0[y,x] - G0[y+1,x] ); Gmax = Math.Max(G, Gmax);
          B = diff( B0[y,x] - B0[y+1,x] ); Bmax = Math.Max(B, Bmax);
          unchecked { imUML.Pixels[pointer] = (Int32)0xff000000 | R << 16 | G << 8 | B; }
          unchecked { imAll.Pixels[pointer] = (Int32)0xff000000 | Rmax << 16 | Gmax << 8 | Bmax; }
        }
      }
    }
  }
}