﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media.Imaging;
using System.Windows.Input;
using System.ComponentModel;

namespace SobelFilterForRealImages
{ public partial class MainPage : UserControl
  { WriteableBitmap img0, imgOut;
    float ContrastFactor = 1f; //contrast enhancement factor
    Int32 xSize, ySize; //image width and height
    Int64 t0, t1;       //system time before and after filtering
    String fileName; 
    Byte [,] R0, G0, B0; //intermediary
    BackgroundWorker myThread = new BackgroundWorker(); //Thread object

    public MainPage()
    { InitializeComponent();
      myThread.DoWork += new DoWorkEventHandler( thread_DoWork );
      myThread.RunWorkerCompleted += 
        new RunWorkerCompletedEventHandler( update_View_when_thread_completed );
      myThread.WorkerSupportsCancellation = true;
    }

    private void myImageIn_ImageOpened( object sender,RoutedEventArgs e ) 
    { img0  = new WriteableBitmap( (BitmapSource)myImageIn.Source );
      fileName = ((BitmapImage)myImageIn.Source).UriSource.OriginalString;
      a_new_image_opened();
    }

    private void myButtonOF_Click( object sender, RoutedEventArgs e )
    { OpenFileDialog openFileDialog = new OpenFileDialog();
      openFileDialog.Filter = "Image Files (*.png,*.jpg)|*.png;*.jpg";
      bool? userClickedOK = openFileDialog.ShowDialog();
      if ( userClickedOK == true )
      { try
        { System.IO.Stream stream = openFileDialog.File.OpenRead();
          fileName = openFileDialog.File.ToString();
          BitmapImage bmp = new BitmapImage();
          bmp.SetSource( stream );
          stream.Close();
          img0 = new WriteableBitmap( bmp );
          a_new_image_opened();
        } catch {}
      }
    }

    private void a_new_image_opened()
    { xSize = img0.PixelWidth;
      ySize = img0.PixelHeight;
      imgOut = new WriteableBitmap( xSize, ySize );
      myImageIn .Source = img0;
      myImageOut.Source = imgOut;
      //prepare_intermediary_arrays
      R0 = new Byte [ySize, xSize];
      G0 = new Byte [ySize, xSize];
      B0 = new Byte [ySize, xSize];
      GC.Collect(); //clean up memory when new arrays override old ones
      for ( int y=0; y < ySize; y++ )
        for ( int x=0; x < xSize; x++ )
        { Int32 c = img0.Pixels[y*xSize+x];
          R0[y,x] = (Byte)((c & 0xff0000) >> 16);
          G0[y,x] = (Byte)((c & 0x00ff00) >>  8);
          B0[y,x] = (Byte) (c & 0x0000ff)       ;
        }
      if ( update_View_before_starting_thread() ) start_myThread();
    }

    private bool update_View_before_starting_thread()
    { myTextBlockOF.Text = fileName + ": " +
          xSize.ToString() + 'x' + ySize.ToString();
      myTextBlockTime.Text = "Wait !"; return true;
    }

    private void start_myThread()
    { if ( myThread.IsBusy ) return;
      myBorder.Cursor = Cursors.Wait;
      t0 = DateTime.Now.Ticks; 
      myThread.RunWorkerAsync();
    }

    private void update_View_when_thread_completed( Object sender,
                                                    RunWorkerCompletedEventArgs e )
    { myImageOut.Source = imgOut;
      myBorder.Cursor = Cursors.Arrow;
      if ( e.Cancelled ) 
      { myTextBlockTime.Text = "";
      }
      else 
      { t1 = DateTime.Now.Ticks;
        myTextBlockTime.Text =  ((t1-t0)/10000).ToString() + " millisec.";
      }
    }

    private void thread_DoWork( Object sender, DoWorkEventArgs e )
    { SobelFilter( e );
    }

    /* The global value "ContrastFactor" is set by this slider of the GUI ********/
    private void Slider_MouseLeftButtonUp( object sender, MouseButtonEventArgs e ) 
    { ContrastFactor = (float)ContrastFactorSlider.Value; //contrast factor
      ContrastFactorTextBox.Text = ContrastFactor.ToString("#.#");
      if ( update_View_before_starting_thread() ) start_myThread();
    }

/******** Here begins the filter algorithm. **************************************/
/******** It would be more professional to write it into a separate class file. **/ 
   
    private void SobelFilter( DoWorkEventArgs e )
    { int Rh, Gh, Bh, Rv, Gv, Bv,  R, G, B, pointer;
      int Rul, Gul, Bul; //upper left              3x3 Kernel scheme:
      int Rum, Gum, Bum; //upper mid                  Xul Xum Xur
      int Rur, Gur, Bur; //upper right                Xml Xmm Xmr
      int Rml, Gml, Bml; //mid   left                 Xll Xlm Xlr
      int Rmm, Gmm, Bmm; //mid   mid 
      int Rmr, Gmr, Bmr; //mid   right
      int Rll, Gll, Bll; //lower left
      int Rlm, Glm, Blm; //lower mid 
      int Rlr, Glr, Blr; //lower right
      ContrastFactor /= 4; //The Sobel result is up to 4*255 (white minus black).
      for ( int y=1; y < ySize-1; y++ )
      { if ( myThread.CancellationPending ) e.Cancel = true; //kill the thread
        pointer = y*xSize; //points now to the first pixel of row y
        for ( int x=1; x < xSize-1; x++, pointer++ )
        { //rename upper kernel row
          Rul = R0[y-1,x-1]; Gul = G0[y-1,x-1]; Bul = B0[y-1,x-1]; 
          Rum = R0[y-1,x  ]; Gum = G0[y-1,x  ]; Bum = B0[y-1,x  ];
          Rur = R0[y-1,x+1]; Gur = G0[y-1,x+1]; Bur = B0[y-1,x+1];
          //rename mid kernel row
          Rml = R0[y  ,x-1]; Gml = G0[y  ,x-1]; Bml = B0[y  ,x-1]; 
          Rmm = R0[y  ,x  ]; Gmm = G0[y  ,x  ]; Bmm = B0[y  ,x  ]; //just for 
          Rmr = R0[y  ,x+1]; Gmr = G0[y  ,x+1]; Bmr = B0[y  ,x+1];
          //rename lower kernel row
          Rll = R0[y+1,x-1]; Gll = G0[y+1,x-1]; Bll = B0[y+1,x-1]; 
          Rlm = R0[y+1,x  ]; Glm = G0[y+1,x  ]; Blm = B0[y+1,x  ];
          Rlr = R0[y+1,x+1]; Glr = G0[y+1,x+1]; Blr = B0[y+1,x+1]; 
          //horizontal Sobel
          Rh = -Rul + Rur - 2*Rml + 2*Rmr - Rll + Rlr;
          Gh = -Gul + Gur - 2*Gml + 2*Gmr - Gll + Glr;
          Bh = -Bul + Bur - 2*Bml + 2*Bmr - Bll + Blr;
          //vertical Sobel
          Rv = -Rul + Rll - 2*Rum + 2*Rlm - Rur + Rlr;
          Gv = -Gul + Gll - 2*Gum + 2*Glm - Gur + Glr;
          Bv = -Bul + Bll - 2*Bum + 2*Blm - Bur + Blr;
          //gradient + contrast + clipping
          R = Math.Min(255, Convert.ToInt32(ContrastFactor * Math.Sqrt(Rh*Rh + Rv*Rv)));
          G = Math.Min(255, Convert.ToInt32(ContrastFactor * Math.Sqrt(Gh*Gh + Gv*Gv)));
          B = Math.Min(255, Convert.ToInt32(ContrastFactor * Math.Sqrt(Bh*Bh + Bv*Bv)));
          //uplod a 32-bit ARGB-pixel
          unchecked { imgOut.Pixels[pointer] = (Int32)0xff000000 | R << 16 | G << 8 | B; }
        }
      }
    }
  }
}
