﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media.Imaging;
using System.Windows.Input;
using System.ComponentModel;

namespace ConvolutionForRealImages 
{ public partial class MainPage : UserControl
  { WriteableBitmap img0, imgOut;
    Int32 xSize, ySize; //image width and height
    float[,] kernel;
    Int64 t0, t1;       //system time before and after filtering
    String fileName; 
    Byte [,] R0, G0, B0; //intermediary
    BackgroundWorker myThread = new BackgroundWorker(); //Thread object

    public MainPage()
    { InitializeComponent();
      myThread.DoWork += new DoWorkEventHandler( thread_DoWork );
      myThread.RunWorkerCompleted += 
        new RunWorkerCompletedEventHandler( update_View_when_thread_completed );
      myThread.WorkerSupportsCancellation = true;
    }

    private void myImageIn_ImageOpened( object sender,RoutedEventArgs e ) 
    { img0  = new WriteableBitmap( (BitmapSource)myImageIn.Source );
      fileName = ((BitmapImage)myImageIn.Source).UriSource.OriginalString;
      a_new_image_opened();
    }

    private void myButtonOF_Click( object sender, RoutedEventArgs e )
    { OpenFileDialog openFileDialog = new OpenFileDialog();
      openFileDialog.Filter = "Image Files (*.png,*.jpg)|*.png;*.jpg";
      bool? userClickedOK = openFileDialog.ShowDialog();
      if ( userClickedOK == true )
      { try
        { System.IO.Stream stream = openFileDialog.File.OpenRead();
          fileName = openFileDialog.File.ToString();
          BitmapImage bmp = new BitmapImage();
          bmp.SetSource( stream );
          stream.Close();
          img0 = new WriteableBitmap( bmp );
          a_new_image_opened();
        } catch {}
      }
    }

    private void a_new_image_opened()
    { xSize = img0.PixelWidth;
      ySize = img0.PixelHeight;
      imgOut = new WriteableBitmap( xSize, ySize );
      myImageIn .Source = img0;
      myImageOut.Source = imgOut;
      //prepare_intermediary_arrays
      R0 = new Byte [ySize, xSize];
      G0 = new Byte [ySize, xSize];
      B0 = new Byte [ySize, xSize];
      GC.Collect(); //clean up memory when new arrays override old ones
      for ( int y=0; y < ySize; y++ )
        for ( int x=0; x < xSize; x++ )
        { Int32 c = img0.Pixels[y*xSize+x];
          R0[y,x] = (Byte)((c & 0xff0000) >> 16);
          G0[y,x] = (Byte)((c & 0x00ff00) >>  8);
          B0[y,x] = (Byte) (c & 0x0000ff)       ;
        }
      if ( update_View_before_starting_thread() ) start_myThread();
    }

    private bool update_View_before_starting_thread()
    { myTextBlockOF.Text = fileName + ": " +
          xSize.ToString() + 'x' + ySize.ToString();
      myTextBlockTime.Text = "Wait !"; return true;
    }

    private void start_myThread()
    { if ( myThread.IsBusy ) return;
      myBorder.Cursor = Cursors.Wait;
      t0 = DateTime.Now.Ticks; 
      myThread.RunWorkerAsync();
    }

    private void update_View_when_thread_completed( Object sender,
                                                    RunWorkerCompletedEventArgs e )
    { myImageOut.Source = imgOut;
      myBorder.Cursor = Cursors.Arrow;
      if ( e.Cancelled ) 
      { myTextBlockTime.Text = "";
      }
      else 
      { t1 = DateTime.Now.Ticks;
        myTextBlockTime.Text =  ((t1-t0)/10000).ToString() + " millisec.";
      }
    }

    private void thread_DoWork( Object sender, DoWorkEventArgs e )
    { Convolution( e );
    }

    private void RBClick(object sender,RoutedEventArgs e)
    { errorText.Text = "";
      switch( ((RadioButton)sender).Content.ToString() )
      { case "Average 3x3": kernelText.Text="1 1 1\r\n" + 
                                            "1 1 1\r\n" + 
                                            "1 1 1";              break;
        case "Fog     5x5": kernelText.Text=" 1  1  1  1  1\r\n" + 
                                            " 1  1  1  1  1\r\n" + 
                                            " 1  1 10  1  1\r\n" + 
                                            " 1  1  1  1  1\r\n" + 
                                            " 1  1  1  1  1";     break;
        case "Gauss   5x5": kernelText.Text=" 2  8 14  8  2\r\n" + 
                                            " 8 37 61 37  8\r\n" + 
                                            "14 61 99 61 14\r\n" + 
                                            " 8 37 61 37  8\r\n" + 
                                            " 2  8 14  8  2";     break;
        case "Lapl H  3x3": kernelText.Text="-1  0  1\r\n" + 
                                            "-2  0  2\r\n" + 
                                            "-1  0  1";           break;
        case "Lapl V  3x3": kernelText.Text="-1 -2 -1\r\n" + 
                                            " 0  0  0\r\n" + 
                                            " 1  2  1";           break;
        case "Empty   7x7": kernelText.Text="0 0 0 0 0 0 0\r\n" + 
                                            "0 0 0 0 0 0 0\r\n" + 
                                            "0 0 0 0 0 0 0\r\n" + 
                                            "0 0 0 0 0 0 0\r\n" + 
                                            "0 0 0 0 0 0 0\r\n" + 
                                            "0 0 0 0 0 0 0\r\n" + 
                                            "0 0 0 0 0 0 0";      break;
        case "Average 7x7": kernelText.Text="1 1 1 1 1 1 1\r\n" + 
                                            "1 1 1 1 1 1 1\r\n" + 
                                            "1 1 1 1 1 1 1\r\n" + 
                                            "1 1 1 1 1 1 1\r\n" + 
                                            "1 1 1 1 1 1 1\r\n" + 
                                            "1 1 1 1 1 1 1\r\n" + 
                                            "1 1 1 1 1 1 1";      break;
      }
    }

    private void ApplyButton_Click( object sender, RoutedEventArgs e )
    { int n = 1;
      foreach (char c in kernelText.Text ) if ( c == '\r' ) n++;
      String[] separator = {" ","\r","\n"};
      String[] s =  (kernelText.Text).Split( separator, StringSplitOptions.RemoveEmptyEntries );
      int m = s.Length / n;
      if ( m*n != s.Length ) { errorText.Text = "Error:\r\nThis isn't a matrix!"; return; }
      if ( m%2 == 0 )        { errorText.Text = "Error:\r\nEven no of columns!" ; return; }
      if ( n%2 == 0 )        { errorText.Text = "Error:\r\nEven no of rows!"    ; return; }
      if ( m < 3 )           { errorText.Text = "Error:\r\nLess than 3 columns!"; return; }
      if ( n < 3 )           { errorText.Text = "Error:\r\nLess than 3 rows!"   ; return; }
      kernel = new float[n,m];
      try 
      { for ( int y=0; y < n; y++ )
         for ( int x=0; x < m; x++ )
          kernel[y,x] = Convert.ToSingle(s[y*m + x]);
      } 
      catch { errorText.Text = "Error:\r\nEnter INTs or FLOATs!"; return; }
      errorText.Text = "m=" + m.ToString() + ", n=" + n.ToString();
      if ( update_View_before_starting_thread() ) start_myThread();
    }

/******** Here begins the convolution algorithm. *********************************/
/******** It would be more professional to write it into a separate class file. **/ 
   
    private void Convolution( DoWorkEventArgs e )
    { if ( kernel == null ) return;
      int m = kernel.GetLength(0); //no of kernel columns
      int n = kernel.GetLength(1); //no of kernel rows
      int mh = (m-1) / 2; //horizontal half wing length
      int nh = (n-1) / 2; //vertical   half wing length
      int i, x, y, xx, yy, xxx, yyy, R, G, B;
      float weight, sumR, sumG, sumB, divisor=0;
      //Add all kernel values to obtain a divisor
      for ( yy=0; yy < n; yy++ )
        for ( xx=0; xx < m; xx++ ) divisor += kernel[yy,xx];
      if ( Math.Abs(divisor) < 1f ) divisor = 1f; //prevents from divison by 0
      //Preset all 32-bit-ARGB-pixels with A = ff and RGB = 000000 
      for ( i=0; i < xSize*ySize; i++ ) unchecked { imgOut.Pixels[i] = (Int32)0xff000000; }
      //Here begins the convolution
      for ( y=nh; y < ySize-nh; y++ )
      { if ( myThread.CancellationPending ) e.Cancel = true; //kill the thread
        i = y*xSize + mh; //linear index of the first pixel in row y
        for ( x=mh; x < xSize-mh; x++, i++ ) //increment x and i
        { sumR = sumG = sumB = 0;
          for ( yy=0; yy < n; yy++ )
          { yyy = y+yy-nh;
            for ( xx=0; xx < m; xx++ )
            { xxx = x+xx-mh;
              weight = kernel[yy,xx];
              sumR += R0[yyy,xxx] * weight;
              sumG += G0[yyy,xxx] * weight;
              sumB += B0[yyy,xxx] * weight;
            }
          }
          //Divide, convert to positive integer and clip to maximal 255 
          R = Math.Min(255, Convert.ToInt32(Math.Abs(sumR/divisor)));
          G = Math.Min(255, Convert.ToInt32(Math.Abs(sumG/divisor)));
          B = Math.Min(255, Convert.ToInt32(Math.Abs(sumB/divisor)));
          //Shift the color bytes to their 32-bit-ARGB positions
          imgOut.Pixels[i] += R << 16 | G << 8 | B;
        }
      }
    }
  }
}

