﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using System.Windows.Input;

namespace ColorHistogram
{ public partial class MainPage : UserControl
  { WriteableBitmap img0, img1; //input+output images
    Int32 xSize, ySize; //image width+height
    String fileName; 
    Int32[,] Histogram = new Int32[3,256];
    Brush[] brush = { new SolidColorBrush( Colors.Red   ),
                      new SolidColorBrush( Colors.Green ),
                      new SolidColorBrush( Colors.Blue  ) };
    Int32[] threshold = { 255, 255, 255 };
    Int32[] R; //intermediary array for red
    Int32[] G; //intermediary array for green
    Int32[] B; //intermediary array for blue
    Canvas[] canvas = new Canvas[3];

    public MainPage()
    { InitializeComponent();
      canvas[0] = RedHistogramCanvas; 
      canvas[1] = GreenHistogramCanvas; 
      canvas[2] = BlueHistogramCanvas; 
    }

    private void myImageIn_ImageOpened( object sender,RoutedEventArgs e ) 
    { //Read "Butterfly.jpg" from the hard disk and remember the string.
      img0  = new WriteableBitmap( (BitmapSource)myImageIn.Source );
      fileName = ((BitmapImage)myImageIn.Source).UriSource.OriginalString;
      a_new_image_opened();
    }

    private void myButtonOF_Click( object sender, RoutedEventArgs e )
    { OpenFileDialog openFileDialog = new OpenFileDialog();
      openFileDialog.Filter = "Image Files (*.png,*.jpg)|*.png;*.jpg";
      bool? userClickedOK = openFileDialog.ShowDialog();
      if ( userClickedOK == true )
      { try //Forget it if the user clicked a strange file.
        { System.IO.Stream stream = openFileDialog.File.OpenRead();
          fileName = openFileDialog.File.ToString();
          BitmapImage bmp = new BitmapImage();
          bmp.SetSource( stream );
          stream.Close();
          img0 = new WriteableBitmap( bmp );
          a_new_image_opened();
        } catch {}
      }
    }

    private void a_new_image_opened()
    { //Create two image clones: RGB=intermediary array; img1 = output
      xSize = img0.PixelWidth;
      ySize = img0.PixelHeight;
      R = new Int32[xSize*ySize];
      G = new Int32[xSize*ySize];
      B = new Int32[xSize*ySize];
      img1 = new WriteableBitmap( xSize, ySize );
      //Inform the user about name and dimensions.
      myTextBlockOF.Text = fileName + ": " + xSize.ToString() + 'x' 
                                           + ySize.ToString();
      ComputeColorHistogram();
      //Set the threshold to 255 and clear the output: ARGB = ff+00+00+00.
      RedSlider.Value = GreenSlider.Value = BlueSlider.Value = 255;
      for ( int i=0; i < xSize*ySize; i++ )
        unchecked { img1.Pixels[i] = (Int32)0xff000000; }
      //Redraw input and output images.
      myImageIn .Source = img0;
      myImageOut.Source = img1;
    }

    private void ComputeColorHistogram()
    { for ( int color=0; color < 3; color++ )
        for ( int gray=0; gray < 256; gray++ ) Histogram[color,gray] = 0; //clear
      for ( int i=0; i < xSize*ySize; i++ ) //for all Int32-pixels
      { Int32 argb = img0.Pixels[i]; //local variables
        R[i] = (argb & 0xff0000) >> 16; //Cut out red   and shift it down.
        G[i] = (argb & 0x00ff00) >>  8; //Cut out green and shift it down.
        B[i] =  argb & 0x0000ff;        //Cut out blue.
        Histogram[0,R[i]]++;         //Increment the histogram bar no. R[i].
        Histogram[1,G[i]]++;         //Increment the histogram bar no. G[i].
        Histogram[2,B[i]]++;         //Increment the histogram bar no. B[i].
      }
      //Scale the tallest bar of the histogram to height = 100.
      Int32 max = 0;
      for ( int color=0; color < 3; color++ )
        for ( int gray=0; gray < 256; gray++ ) 
          if ( Histogram[color,gray] > max ) max = Histogram[color,gray];
      float factor = 100f / max;
      for ( int color=0; color < 3; color++ )
      { for ( int gray=0; gray < 256; gray++ ) 
          Histogram[color,gray] = Convert.ToInt32(Histogram[color,gray] * factor);
        canvas[color].Children.Clear(); //Throw away any former canvas content.
        //Draw 256 vertical bars on the canvas.
        for ( int gray=0; gray < 256; gray++ )
        { Line line = new Line(); line.X1 = line.X2 = gray; 
          line.Y1 = 100; line.Y2 = 100 - Histogram[color,gray];
          line.Stroke = brush[color]; line.StrokeThickness = 1; 
          canvas[color].Children.Add( line );
        }
      }
    } 

    private void Slider_MouseLeftButtonUp( object sender, MouseButtonEventArgs e ) 
    { switch( ((Slider)sender).Name )
      { case "RedSlider"  : threshold[0] = (Int32)RedSlider  .Value; break;
        case "GreenSlider": threshold[1] = (Int32)GreenSlider.Value; break;
        case "BlueSlider" : threshold[2] = (Int32)BlueSlider .Value; break;
      }
      for ( int i=0; i < xSize*ySize; i++ )
      { int argb; unchecked { argb = (Int32)0xff000000; } //ARGB = ff+00+00+00 = black.
        if ( R[i] > threshold[0] ) argb += 0x00ff0000;    //ARGB = ff+ff+00+00 = red. 
        if ( G[i] > threshold[1] ) argb += 0x0000ff00;    //ARGB = ff+00+ff+00 = green. 
        if ( B[i] > threshold[2] ) argb += 0x000000ff;    //ARGB = ff+00+00+ff = blue. 
        img1.Pixels[i] = argb; //Copy argb to its final pixel position.
      }
      myImageOut.Source = img1; //Display the output image on the screen.
    }
  }//end of class MainPage
}//end of namespace

