﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using System.Windows.Input;

namespace GrayHistogram
{ public partial class MainPage : UserControl
  { WriteableBitmap img0, img1; //input+output images
    Int32 xSize, ySize; //image width+height
    String fileName; 
    Int32[] Histogram = new Int32[256];
    Brush brush = new SolidColorBrush( Colors.Black );
    Int32 threshold = 255;
    Int32[] RGB; //intermediary array for (r+g+b)/3

    public MainPage()
    { InitializeComponent(); 
    }

    private void myImageIn_ImageOpened( object sender,RoutedEventArgs e ) 
    { //Read "Lena256.png" from the hard disk and remember the string.
      img0  = new WriteableBitmap( (BitmapSource)myImageIn.Source );
      fileName = ((BitmapImage)myImageIn.Source).UriSource.OriginalString;
      a_new_image_opened();
    }

    private void myButtonOF_Click( object sender, RoutedEventArgs e )
    { OpenFileDialog openFileDialog = new OpenFileDialog();
      openFileDialog.Filter = "Image Files (*.png,*.jpg)|*.png;*.jpg";
      bool? userClickedOK = openFileDialog.ShowDialog();
      if ( userClickedOK == true )
      { try //Forget it if the user clicked a strange file.
        { System.IO.Stream stream = openFileDialog.File.OpenRead();
          fileName = openFileDialog.File.ToString();
          BitmapImage bmp = new BitmapImage();
          bmp.SetSource( stream );
          stream.Close();
          img0 = new WriteableBitmap( bmp );
          a_new_image_opened();
        } catch {}
      }
    }

    private void a_new_image_opened()
    { //Create two image clones: RGB=intermediary array; img1 = output
      xSize = img0.PixelWidth;
      ySize = img0.PixelHeight;
      RGB = new Int32[xSize*ySize];
      img1 = new WriteableBitmap( xSize, ySize );
      //Inform the user about name and dimensions.
      myTextBlockOF.Text = fileName + ": " + xSize.ToString() + 'x' 
                                           + ySize.ToString();
      ComputeColorHistogram();
      //Set the threshold to 255 and clear the output: ARGB = ff+00+00+00.
      Slider.Value = 255;
      for ( int i=0; i < xSize*ySize; i++ )
        unchecked { img1.Pixels[i] = (Int32)0xff000000; }
      //Redraw input and output images.
      myImageIn .Source = img0;
      myImageOut.Source = img1;
    }

    private void ComputeColorHistogram()
    { for ( int gray=0; gray < 256; gray++ ) Histogram[gray] = 0; //clear
      for ( int i=0; i < xSize*ySize; i++ ) //for all Int32-pixels
      { Int32 r, g ,b, argb = img0.Pixels[i]; //local variables
        r = (argb & 0xff0000) >> 16; //Cut out red   and shift it down.
        g = (argb & 0x00ff00) >>  8; //Cut out green and shift it down.
        b =  argb & 0x0000ff;        //Cut out blue.
        RGB[i] = (r + g + b) / 3;    //gray = mean of r+g+b
        Histogram[RGB[i]]++;         //Increment the histogram bar no. RGB[i].
      }
      //Scale the tallest bar of the histogram to height = 100.
      Int32 max = 0;
      for ( int gray=0; gray < 256; gray++ ) 
        if ( Histogram[gray] > max ) max = Histogram[gray];
      float factor = 100f / max;
      for ( int gray=0; gray < 256; gray++ ) 
        Histogram[gray] = Convert.ToInt32(Histogram[gray] * factor);
      HistogramCanvas.Children.Clear(); //Throw away any former canvas content.
      //Draw 256 vertical bars on the canvas.
      for ( int gray=0; gray < 256; gray++ )
      { Line line = new Line(); line.X1 = line.X2 = gray; 
        line.Y1 = 100; line.Y2 = 100 - Histogram[gray];
        line.Stroke = brush; line.StrokeThickness = 1; 
        HistogramCanvas.Children.Add( line );
      }
    } 

    private void Slider_MouseLeftButtonUp( object sender, MouseButtonEventArgs e ) 
    { threshold = (Int32)Slider.Value;
      for ( int i=0; i < xSize*ySize; i++ )
      { int argb; unchecked { argb = (Int32)0xff000000; } //ARGB = ff+00+00+00 = black.
        if ( RGB[i] > threshold ) argb += 0x00ffffff;     //ARGB = ff+ff+ff+ff = white. 
        img1.Pixels[i] = argb; //Copy argb to its final pixel position.
      }
      myImageOut.Source = img1; //Display the output image on the screen.
    }
  }//end of class MainPage
}//end of namespace DCT2DColor

